import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { memo, useState, useMemo, useCallback } from 'react';
import classNames from 'classnames';
import { mapValues } from 'lodash';
import { getChangeKey, getChangeLineNum } from '../../utils';
import CodeCell from '../CodeCell';
import { composeCallback, renderDefaultBy, wrapInAnchorBy } from '../utils';
import dayjs from 'dayjs';
const SIDE_OLD = 0;
const SIDE_NEW = 1;
function useCallbackOnSide(side, setHover, change, customCallbacks) {
    const markHover = useCallback(() => setHover(side), [side, setHover]);
    const unmarkHover = useCallback(() => setHover(''), [setHover]);
    // Unlike selectors, hooks do not provide native functionality to customize comparator,
    // on realizing that this does not reduce amount of renders, only preventing duplicate merge computations,
    // we decide not to optimize this extremely, leave it recomputed on certain rerenders.
    const callbacks = useMemo(() => {
        const callbacks = mapValues(customCallbacks, fn => (e) => fn && fn({ side, change }, e));
        callbacks.onMouseEnter = composeCallback(markHover, callbacks.onMouseEnter);
        callbacks.onMouseLeave = composeCallback(unmarkHover, callbacks.onMouseLeave);
        return callbacks;
    }, [change, customCallbacks, markHover, side, unmarkHover]);
    return callbacks;
}
function renderCells(args) {
    const { change, side, selected, tokens, gutterClassName, codeClassName, gutterEvents, codeEvents, anchorID, gutterAnchor, gutterAnchorTarget, hideGutter, hover, renderToken, renderGutter, blameLines, blameVisible, handleMouseEnter, handleMouseLeave, } = args;
    // 先留着这个
    const blameProps = classNames('diff-gutter', 'diff-gutter-normal', 'diff-blame');
    const showBlame = blameLines && blameLines?.length && blameLines.length > 0 || undefined;
    if (!change) {
        const gutterClassNameValue = classNames('diff-gutter', 'diff-gutter-omit', gutterClassName);
        const codeClassNameValue = classNames('diff-code', 'diff-code-omit', codeClassName);
        // 这里加上 blame 信息
        return [
            !hideGutter && showBlame && (_jsx("td", { className: blameProps }, "blame")),
            !hideGutter && _jsx("td", { className: gutterClassNameValue }, "gutter"),
            _jsx("td", { className: codeClassNameValue }, "code"),
        ];
    }
    const { type, content } = change;
    const changeKey = getChangeKey(change);
    const changeNo = getChangeLineNum(change);
    const sideName = side === SIDE_OLD ? 'old' : 'new';
    const gutterClassNameValue = classNames('diff-gutter', `diff-gutter-${type}`, {
        'diff-gutter-selected': selected,
        ['diff-line-hover-' + sideName]: hover,
    }, gutterClassName);
    const gutterOptions = {
        change,
        side: sideName,
        inHoverState: hover,
        renderDefault: renderDefaultBy(change, sideName),
        wrapInAnchor: wrapInAnchorBy(gutterAnchor, gutterAnchorTarget),
    };
    const gutterProps = {
        id: anchorID || undefined,
        className: gutterClassNameValue,
        children: renderGutter(gutterOptions),
        ...gutterEvents,
    };
    const codeClassNameValue = classNames('diff-code', `diff-code-${type}`, {
        'diff-code-selected': selected,
        ['diff-line-hover-' + sideName]: hover,
    }, codeClassName);
    // eslint-disable-next-line no-console
    // console.log('spilt renderCells:', blameLines);
    const findBlameItem = blameLines?.find(item => changeNo >= item.commit.start_line && changeNo <= item.commit.end_line);
    // eslint-disable-next-line no-console
    // console.log('spilt renderCells:', findBlameItem);
    return [
        // 这里加上blame信息
        findBlameItem && (_jsxs("td", { onMouseEnter: handleMouseEnter, onMouseLeave: handleMouseLeave, className: blameProps, children: [findBlameItem.commit.committer_name, blameVisible && (_jsxs("div", { style: {
                        position: 'absolute',
                        top: '-5px',
                        left: '135px',
                        color: '#999',
                        backgroundColor: '#fff',
                        zIndex: 100,
                        padding: '5px 5px',
                        width: 'auto',
                        borderRadius: '6px',
                        bottom: 'auto',
                        marginLeft: 'auto',
                        textAlign: 'left',
                        transition: 'opacity 0.3s',
                    }, children: ["\u884C\u53F7\uFF1A", changeNo, _jsx("br", {}), "\u63D0\u4EA4ID\uFF1A", findBlameItem?.commit?.id, _jsx("br", {}), "\u63D0\u4EA4\u4EBA\uFF1A", findBlameItem?.commit?.committer_name, _jsx("br", {}), "\u63D0\u4EA4\u65E5\u671F\uFF1A", dayjs(findBlameItem?.commit?.committed_date).format('YYYY-MM-DD HH:mm:ss')] }))] }, `blame${changeNo}`)),
        !hideGutter && _jsx("td", { ...gutterProps, "data-change-key": changeKey }, "gutter"),
        _jsx(CodeCell, { className: codeClassNameValue, changeKey: changeKey, text: content, tokens: tokens, renderToken: renderToken, ...codeEvents }, "code"),
    ];
}
function SplitChange(props) {
    const { className, oldChange, newChange, oldSelected, newSelected, oldTokens, newTokens, monotonous, gutterClassName, codeClassName, gutterEvents, codeEvents, hideGutter, generateAnchorID, gutterAnchor, renderToken, renderGutter, oldBlameLines, newBlameLines, } = props;
    const [isVisible, setIsVisible] = useState(false);
    const [hover, setHover] = useState('');
    const oldGutterEvents = useCallbackOnSide('old', setHover, oldChange, gutterEvents);
    const newGutterEvents = useCallbackOnSide('new', setHover, newChange, gutterEvents);
    const oldCodeEvents = useCallbackOnSide('old', setHover, oldChange, codeEvents);
    const newCodeEvents = useCallbackOnSide('new', setHover, newChange, codeEvents);
    const oldAnchorID = oldChange && generateAnchorID(oldChange);
    const newAnchorID = newChange && generateAnchorID(newChange);
    const handleMouseEnter = () => {
        // 获取鼠标位置
        // const {clientX, clientY} = event;
        // setPosition({top: clientY, left: clientX + 10}); // 在光标右侧显示，增加 10 像素的偏移
        setIsVisible(true);
    };
    const handleMouseLeave = () => {
        setIsVisible(false);
    };
    const commons = {
        monotonous,
        hideGutter,
        gutterClassName,
        codeClassName,
        gutterEvents,
        codeEvents,
        renderToken,
        renderGutter,
    };
    const oldArgs = {
        ...commons,
        change: oldChange,
        side: SIDE_OLD,
        selected: oldSelected,
        tokens: oldTokens,
        gutterEvents: oldGutterEvents,
        codeEvents: oldCodeEvents,
        anchorID: oldAnchorID,
        gutterAnchor: gutterAnchor,
        gutterAnchorTarget: oldAnchorID,
        hover: hover === 'old',
        blameLines: oldBlameLines,
        blameVisible: isVisible,
        handleMouseEnter,
        handleMouseLeave,
    };
    const newArgs = {
        ...commons,
        change: newChange,
        side: SIDE_NEW,
        selected: newSelected,
        tokens: newTokens,
        gutterEvents: newGutterEvents,
        codeEvents: newCodeEvents,
        anchorID: oldChange === newChange ? null : newAnchorID,
        gutterAnchor: gutterAnchor,
        gutterAnchorTarget: oldChange === newChange ? oldAnchorID : newAnchorID,
        hover: hover === 'new',
        blameLines: newBlameLines,
        blameVisible: isVisible,
        handleMouseEnter,
        handleMouseLeave,
    };
    if (monotonous) {
        return (_jsx("tr", { className: classNames('diff-line', className), children: renderCells(oldChange ? oldArgs : newArgs) }));
    }
    const lineTypeClassName = ((oldChange, newChange) => {
        if (oldChange && !newChange) {
            return 'diff-line-old-only';
        }
        if (!oldChange && newChange) {
            return 'diff-line-new-only';
        }
        if (oldChange === newChange) {
            return 'diff-line-normal';
        }
        return 'diff-line-compare';
    })(oldChange, newChange);
    return (_jsxs("tr", { className: classNames('diff-line', lineTypeClassName, className), children: [renderCells(oldArgs), renderCells(newArgs)] }));
}
export default memo(SplitChange);
//# sourceMappingURL=SplitChange.js.map